import logging
import random
from typing import Dict, List, Tuple, cast

from faker import Faker
from locust import events, task

from performance_tests.common import (
    ProvisioningClient,
    ResponseError,
    get_id_broker_classes,
    get_provisioning_api_admins,
)

user_credentials: List[Tuple[str, str]] = []  # [(username, password), ..]
id_broker_school_classes: Dict[str, Dict[str, Dict[str, str]]] = {}  # {sch_auth: {ou: {cl: uuid, ...}}}
fake = Faker()


@events.init.add_listener
def on_init(environment, **kwargs):
    creds = get_provisioning_api_admins()
    credentials = cast(List[Tuple[str, str]], sorted(creds.items()))
    user_credentials.extend(credentials)
    id_broker_school_classes.update(get_id_broker_classes())


class ProvisioningAPIClient(ProvisioningClient):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.auth_username, self.auth_password = random.choice(user_credentials)
        self.school_authority = self.auth_username.rsplit("-", 1)[-1]
        self.schools = cast(
            List[Tuple[str, Dict[str, str]]],
            list(id_broker_school_classes[self.school_authority].items()),
        )

    @task
    def school_class_update(self) -> None:
        school, classes = random.choice(self.schools)
        class_name, entry_uuid = random.choice(list(classes.items()))
        try:
            super().school_class_update(
                school_authority=self.school_authority,
                obj_id=entry_uuid,
                name=class_name,
                school=school,
                description=f"Performance Test temp class modified {fake.pyint(1000, 9999)}",
                members=[],
            )
        except ResponseError as exc:
            logging.error(
                "%s: school_authority=%r name=%r entry_uuid=%r -> %r",
                self.greenlet.name,
                self.school_authority,
                class_name,
                entry_uuid,
                exc,
            )
